@@ -13,6 +13,7 @@ from keg_auth import model
 from keg_auth.libs.authenticators import (
     DefaultPasswordPolicy,
     KegAuthenticator,
+    OAuthAuthenticator,
 )
 
 DEFAULT_CRYPTO_SCHEMES = ('bcrypt', 'pbkdf2_sha256',)
@@ -52,12 +53,15 @@ class AuthManager(object):
         'after-reset': '{blueprint}.login',
         'verify-account': '{blueprint}.verify-account',
         'after-verify-account': '{blueprint}.login',
+        'oauth-login': '{blueprint}.oauth-login',
+        'oauth-authorize': '{blueprint}.oauth-authorize',
     }
     cli_group_name = 'auth'
 
     def __init__(self, mail_manager=None, blueprint='auth', endpoints=None,
                  cli_group_name=None, grid_cls=None, login_authenticator=KegAuthenticator,
                  request_loaders=None, permissions=None, entity_registry=None,
+                 oauth_authenticator=OAuthAuthenticator,
                  password_policy_cls=DefaultPasswordPolicy):
         self.mail_manager = mail_manager
         self.blueprint_name = blueprint
@@ -70,6 +74,7 @@ class AuthManager(object):
         self.cli_group = None
         self.grid_cls = grid_cls
         self.login_authenticator_cls = login_authenticator
+        self.oauth_authenticator_cls = oauth_authenticator
         self.request_loader_cls = tolist(request_loaders or [])
         self.request_loaders = dict()
         self.menus = dict()
@@ -116,6 +121,12 @@ class AuthManager(object):
         # Use select2 for form selects in templates extending keg_auth/form-base.
         app.config.setdefault('KEGAUTH_USE_SELECT2', True)
 
+        # Set the login target for the redirect authenticator
+        app.config.setdefault('KEGAUTH_REDIRECT_LOGIN_TARGET', None)
+
+        # OAuth profiles
+        app.config.setdefault('KEGAUTH_OAUTH_PROFILES', [])
+
         # Set defaults for OIDC URI locations
         app.config.setdefault('OIDC_AUTH_URI', '/oauth2/v1/authorize')
         app.config.setdefault('OIDC_TOKEN_URI', '/oauth2/v1/token')
@@ -124,20 +135,27 @@ class AuthManager(object):
         app.config.setdefault('KEGAUTH_OIDC_LOGOUT_REDIRECT', None)
 
         # Attempt lockout parameters.
+        # - Enabled: default True, turns on attempt limits and requires the attempt entity.
         # - Limit: maximum number of attempts within the timespan.
         # - Timespan: number of seconds in which the limit can be reached.
         # - Lockout: number of seconds until an attempt can be made after the limit is reached.
         # - KEGAUTH_ATTEMPT_IP_LIMIT: base locking on IP address as well as input
-        app.config.setdefault('KEGAUTH_LOGIN_ATTEMPT_LIMIT', 3)
-        app.config.setdefault('KEGAUTH_LOGIN_ATTEMPT_TIMESPAN', 3600)  # 1 hour
-        app.config.setdefault('KEGAUTH_LOGIN_ATTEMPT_LOCKOUT', 3600)  # 1 hour
-        app.config.setdefault('KEGAUTH_FORGOT_ATTEMPT_LIMIT', 5)
-        app.config.setdefault('KEGAUTH_FORGOT_ATTEMPT_TIMESPAN', 3600)  # 1 hour
-        app.config.setdefault('KEGAUTH_FORGOT_ATTEMPT_LOCKOUT', 3600)  # 1 hour
-        app.config.setdefault('KEGAUTH_RESET_ATTEMPT_LIMIT', 1)
-        app.config.setdefault('KEGAUTH_RESET_ATTEMPT_TIMESPAN', 86400)  # 24 hours
-        app.config.setdefault('KEGAUTH_RESET_ATTEMPT_LOCKOUT', 86400)  # 24 hours
-        app.config.setdefault('KEGAUTH_ATTEMPT_IP_LIMIT', False)
+        app.config.setdefault('KEGAUTH_ATTEMPT_LIMIT_ENABLED', True)
+        app.config.setdefault('KEGAUTH_ATTEMPT_LIMIT', 15)
+        app.config.setdefault('KEGAUTH_ATTEMPT_TIMESPAN', 600)  # 10 minutes
+        app.config.setdefault('KEGAUTH_ATTEMPT_LOCKOUT', 3600)  # 1 hour
+        app.config.setdefault('KEGAUTH_ATTEMPT_IP_LIMIT', True)
+        # Lockout parameters may also be set for specific views (login, forgot, reset). If these
+        # specific values are not set, the generic ATTEMPT values will be used.
+        # app.config.setdefault('KEGAUTH_LOGIN_ATTEMPT_LIMIT', 3)
+        # app.config.setdefault('KEGAUTH_LOGIN_ATTEMPT_TIMESPAN', 3600)  # 1 hour
+        # app.config.setdefault('KEGAUTH_LOGIN_ATTEMPT_LOCKOUT', 3600)  # 1 hour
+        # app.config.setdefault('KEGAUTH_FORGOT_ATTEMPT_LIMIT', 5)
+        # app.config.setdefault('KEGAUTH_FORGOT_ATTEMPT_TIMESPAN', 3600)  # 1 hour
+        # app.config.setdefault('KEGAUTH_FORGOT_ATTEMPT_LOCKOUT', 3600)  # 1 hour
+        # app.config.setdefault('KEGAUTH_RESET_ATTEMPT_LIMIT', 1)
+        # app.config.setdefault('KEGAUTH_RESET_ATTEMPT_TIMESPAN', 86400)  # 24 hours
+        # app.config.setdefault('KEGAUTH_RESET_ATTEMPT_LOCKOUT', 86400)  # 24 hours
 
     def init_cli(self, app):
         """Add a CLI group for auth."""
@@ -186,6 +204,9 @@ class AuthManager(object):
 
         self.login_authenticator = self.login_authenticator_cls(app)
 
+        if app.config.get('KEGAUTH_OAUTH_PROFILES'):
+            self.oauth_authenticator = OAuthAuthenticator(app)
+
         for loader_cls in self.request_loader_cls:
             self.request_loaders[loader_cls.get_identifier()] = loader_cls(app)
 
@@ -350,7 +371,11 @@ class AuthManager(object):
         # which may not be available earlier
         user.token_generate()
 
-        if mail_enabled and self.mail_manager:
+        if (
+            mail_enabled
+            and self.mail_manager
+            and not self.login_authenticator.is_domain_excluded(user.username)
+        ):
             self.mail_manager.send_new_user(user)
 
         # use add + commit here instead of user_class.add() above so the user isn't actually
