@@ -9,11 +9,13 @@ import sqlalchemy as sa
 import string
 import typing
 import wtforms
+from blazeutils import tolist
 from keg.db import db
 from sqlalchemy_utils import EmailType
 
 from keg_auth import forms
 from keg_auth.extensions import flash, lazy_gettext as _
+from keg_auth.libs import get_domain_from_email
 from keg_auth.model import get_username_key, get_username
 from keg_auth.model.entity_registry import RegistryError
 
@@ -83,6 +85,9 @@ class LoginAuthenticator(object):
     def get_responder(self, key):
         return self.responders.get(key)
 
+    def is_domain_excluded(self, login_id):
+        return False
+
 
 class ViewResponder(object):
     """ View-layer logic wrapper for use in the Authenticator
@@ -236,10 +241,15 @@ class AttemptLimitMixin(object):
         return flask.current_app.auth_manager.entity_registry.attempt_cls
 
     def should_limit_attempts(self):
-        try:
-            return self.attempt_ent is not None
-        except RegistryError:
+        if not flask.current_app.config.get('KEGAUTH_ATTEMPT_LIMIT_ENABLED'):
+            # limiting can be turned off by config
             return False
+        try:
+            # verify the attempt entity has been configured for logging attempts
+            self.attempt_ent
+        except RegistryError:
+            raise Exception('Rate limiting is enabled, but the attempt entity is not registered')
+        return True
 
     @property
     def should_filter_ip(self):
@@ -320,17 +330,34 @@ class AttemptLimitMixin(object):
     def get_flash_attempts_limit_reached(self):
         raise NotImplementedError  # pragma: no cover
 
-    def get_attempt_limit(self):
-        raise NotImplementedError  # pragma: no cover
-
-    def get_attempt_timespan(self):
-        raise NotImplementedError  # pragma: no cover
-
     def get_attempt_type(self):
         raise NotImplementedError  # pragma: no cover
 
+    def get_config_value(self, key):
+        """Fetch the config value for the view specified in get_attempt_type.
+
+        Checks the view-specific key first, and if the value is not configured,
+        falls to the default for the key.
+
+        Example: get_attempt_type returns login, and key is attempt_limit.
+        - Checks for KEGAUTH_LOGIN_ATTEMPT_LIMIT first
+        - If that is not set, falls back to KEGAUTH_ATTEMPT_LIMIT
+        """
+        view_key = self.get_attempt_type().upper()
+        key = key.upper()
+        return flask.current_app.config.get(
+            f'KEGAUTH_{view_key}_{key}',
+            flask.current_app.config.get(f'KEGAUTH_{key}')
+        )
+
+    def get_attempt_limit(self):
+        return self.get_config_value('attempt_limit')
+
+    def get_attempt_timespan(self):
+        return self.get_config_value('attempt_timespan')
+
     def get_attempt_lockout_period(self):
-        raise NotImplementedError  # pragma: no cover
+        return self.get_config_value('attempt_lockout')
 
 
 class PasswordSetterResponderBase(FormResponderMixin, ViewResponder):
@@ -411,15 +438,6 @@ class ResetPasswordViewResponder(AttemptLimitMixin, PasswordSetterResponderBase)
     def get_flash_attempts_limit_reached(self):
         return _('Too many password reset attempts.'), 'error'
 
-    def get_attempt_limit(self):
-        return flask.current_app.config.get('KEGAUTH_RESET_ATTEMPT_LIMIT')
-
-    def get_attempt_timespan(self):
-        return flask.current_app.config.get('KEGAUTH_RESET_ATTEMPT_TIMESPAN')
-
-    def get_attempt_lockout_period(self):
-        return flask.current_app.config.get('KEGAUTH_RESET_ATTEMPT_LOCKOUT')
-
     def get_attempt_type(self):
         return 'reset'
 
@@ -508,15 +526,6 @@ class PasswordFormViewResponder(AttemptLimitMixin, LoginResponderMixin,
     def get_flash_attempts_limit_reached(self):
         return _('Too many failed login attempts.'), 'error'
 
-    def get_attempt_limit(self):
-        return flask.current_app.config.get('KEGAUTH_LOGIN_ATTEMPT_LIMIT')
-
-    def get_attempt_timespan(self):
-        return flask.current_app.config.get('KEGAUTH_LOGIN_ATTEMPT_TIMESPAN')
-
-    def get_attempt_lockout_period(self):
-        return flask.current_app.config.get('KEGAUTH_LOGIN_ATTEMPT_LOCKOUT')
-
     def get_attempt_type(self):
         return 'login'
 
@@ -561,6 +570,63 @@ class PasswordFormViewResponder(AttemptLimitMixin, LoginResponderMixin,
         ).count()
 
 
+class OAuthLoginViewResponder(ViewResponder):
+    """ OAuth logins, using a provider via authlib"""
+    url = '/login/<name>'
+
+    def get(self, *args, **kwargs):
+        name = kwargs.get('name')
+        oauth = flask.current_app.auth_manager.oauth
+        client = oauth.create_client(name)
+        if not client:
+            return flask.abort(404)
+
+        redirect_uri = flask.current_app.auth_manager.url_for(
+            'oauth-authorize', name=name, _external=True
+        )
+        return client.authorize_redirect(redirect_uri)
+
+    def head(self, *args, **kwargs):
+        return flask.abort(405, valid_methods=['GET'])
+
+    def post(self, *args, **kwargs):
+        return flask.abort(405, valid_methods=['GET'])
+
+
+class OAuthAuthorizeViewResponder(LoginResponderMixin, ViewResponder):
+    """ OAuth logins, using a provider via authlib"""
+    url = '/oauth-authorize/<name>'
+    template_name = 'keg_auth/flash-messages-only.html'
+
+    def get(self, *args, **kwargs):
+        name = kwargs.get('name')
+        oauth = flask.current_app.auth_manager.oauth
+        client = oauth.create_client(name)
+        if not client:
+            return flask.abort(404)
+
+        oauth_profile = self.parent.select_oauth_profile(name)
+        token = client.authorize_access_token()
+        userinfo = token.get('userinfo')
+        if not userinfo:
+            userinfo = client.userinfo()
+
+        login_id = userinfo.get(oauth_profile['id_field'])
+
+        try:
+            user = self.parent.verify_user(profile_name=name, login_id=login_id)
+
+            # User is active and password is verified
+            return self.on_success(user)
+        except UserNotFound:
+            self.on_invalid_user(login_id)
+        except UserInactive as exc:
+            self.on_inactive_user(exc.user)
+
+    def head(self, *args, **kwargs):
+        return flask.abort(405, valid_methods=['GET'])
+
+
 class ForgotPasswordViewResponder(AttemptLimitMixin, UserResponderMixin, FormResponderMixin,
                                   ViewResponder):
     """ Master responder for keg-integrated logins, using an email form"""
@@ -615,15 +681,6 @@ class ForgotPasswordViewResponder(AttemptLimitMixin, UserResponderMixin, FormRes
     def get_flash_attempts_limit_reached(self):
         return _('Too many failed attempts.'), 'error'
 
-    def get_attempt_limit(self):
-        return flask.current_app.config.get('KEGAUTH_FORGOT_ATTEMPT_LIMIT')
-
-    def get_attempt_timespan(self):
-        return flask.current_app.config.get('KEGAUTH_FORGOT_ATTEMPT_TIMESPAN')
-
-    def get_attempt_lockout_period(self):
-        return flask.current_app.config.get('KEGAUTH_FORGOT_ATTEMPT_LOCKOUT')
-
     def get_attempt_type(self):
         return 'forgot'
 
@@ -680,6 +737,26 @@ class LogoutViewResponder(ViewResponder):
         flask.abort(flask.redirect(redirect_to))
 
 
+def not_found_view_responder_factory(target_url):
+    class NotFoundViewResponder(ViewResponder):
+        url = target_url
+
+        def get(self):
+            flask.abort(404)
+
+    return NotFoundViewResponder
+
+
+class RedirectLoginViewResponder(ViewResponder):
+    url = '/login'
+
+    def get(self):
+        target = flask.current_app.config.get('KEGAUTH_REDIRECT_LOGIN_TARGET')
+        if not target or not target.startswith('/'):
+            raise Exception('KEGAUTH_REDIRECT_LOGIN_TARGET not set or not relative')
+        return flask.redirect(target)
+
+
 class PasswordAuthenticatorMixin(object):
     """ Username/password authenticators will need a way to verify a user is valid
         prior to making it the current user in flask login """
@@ -703,6 +780,21 @@ class TokenLoaderMixin(object):
         raise NotImplementedError
 
 
+class RedirectAuthenticator(LoginAuthenticator):
+    """ Redirects to another source for authentication. Useful for when we have an OAuth source
+        in mind for primary auth. We will want to redirect /login there, keep /logout, and
+        direct other responder keys to return 404.
+
+        Use KEGAUTH_REDIRECT_LOGIN_TARGET to set the login target."""
+    responder_cls = {
+        'login': RedirectLoginViewResponder,
+        'forgot-password': not_found_view_responder_factory('/forgot-password'),
+        'reset-password': not_found_view_responder_factory('/reset-password'),
+        'verify-account': not_found_view_responder_factory('/verify-account'),
+        'logout': LogoutViewResponder,
+    }
+
+
 class KegAuthenticator(PasswordAuthenticatorMixin, LoginAuthenticator):
     """ Uses username/password authentication with a login form, validates against keg-auth db"""
     responder_cls = {
@@ -713,7 +805,32 @@ class KegAuthenticator(PasswordAuthenticatorMixin, LoginAuthenticator):
         'logout': LogoutViewResponder,
     }
 
+    def __init__(self, app):
+        super().__init__(app)
+
+        self.load_domain_exclusions(app)
+
+    def load_domain_exclusions(self, app):
+        domain_exclusions = []
+        for profile in app.config.get('KEGAUTH_OAUTH_PROFILES', []):
+            domain_exclusions.extend(tolist(profile.get('domain_filter', [])))
+        self.domain_exclusions = domain_exclusions
+
+    def is_domain_excluded(self, login_id):
+        """Domains configured for OAuth access are excluded from the password authenticator.
+
+        Any operations not using ``verify_user`` should check for exclusion."""
+        if self.domain_exclusions:
+            domain = get_domain_from_email(login_id)
+            if domain and domain in self.domain_exclusions:
+                return True
+        return False
+
     def verify_user(self, login_id=None, password=None, allow_unverified=False):
+        if self.is_domain_excluded(login_id):
+            # apply a domain filter before looking up the user record
+            raise UserNotFound
+
         user = self.user_ent.query.filter_by(username=login_id).one_or_none()
 
         if not user:
@@ -731,6 +848,70 @@ class KegAuthenticator(PasswordAuthenticatorMixin, LoginAuthenticator):
         return user.password == password
 
 
+class OAuthAuthenticator(LoginAuthenticator):
+    """ Uses OAuth authentication via authlib, validates user info against keg-auth db"""
+    responder_cls = {
+        'login': OAuthLoginViewResponder,
+        'authorize': OAuthAuthorizeViewResponder,
+    }
+
+    def __init__(self, app):
+        self.load_profiles(app)
+        super().__init__(app)
+
+    def load_profiles(self, app):
+        from authlib.integrations.flask_client import OAuth
+
+        app.auth_manager.oauth = oauth = OAuth(app)
+        for profile in app.config.get('KEGAUTH_OAUTH_PROFILES'):
+            expected_keys = 'domain_filter', 'id_field', 'oauth_client_kwargs'
+            if len(set(expected_keys) & set(profile.keys())) != len(expected_keys):
+                expected_keys_display = ', '.join(expected_keys)
+                raise Exception(
+                    f'OAuth profile is missing keys, expects {expected_keys_display}'
+                )
+            oauth.register(**profile['oauth_client_kwargs'])
+
+    def select_oauth_profile(self, name):
+        return next(
+            filter(
+                lambda profile: profile['oauth_client_kwargs']['name'] == name,
+                flask.current_app.config.get('KEGAUTH_OAUTH_PROFILES', [])
+            ), None
+        )
+
+    def verify_user(self, profile_name=None, login_id=None):
+        oauth_profile = self.select_oauth_profile(profile_name)
+        if not oauth_profile:
+            raise Exception(f'OAuth profile {profile_name} is not configured')
+
+        # Some OAuth providers, like Google, can be configured to only pass through users
+        #   from specific domains. But we shouldn't make that assumption for all providers,
+        #   and since we have the domain filter also for disallowing logins in KegAuthenticator,
+        #   it's best to check it here as well.
+        if oauth_profile.get('domain_filter'):
+            domain = get_domain_from_email(login_id)
+            if domain and domain not in tolist(oauth_profile['domain_filter']):
+                raise UserNotFound
+
+        user = self.user_ent.query.filter_by(username=login_id).one_or_none()
+
+        if not user:
+            raise UserNotFound
+
+        # Users from domains filtered to OAuth do not follow the usual verification
+        #   process. Check for that here - if OAuth source says it's a verified user,
+        #   trust it.
+        if not user.is_verified:
+            user.is_verified = True
+            db.session.commit()
+
+        if not user.is_active:
+            raise UserInactive(user)
+
+        return user
+
+
 class LdapAuthenticator(KegAuthenticator):
     """ Uses username/password authentication with a login form, validates against LDAP host
 
@@ -800,7 +981,7 @@ class OidcLoginViewResponder(LoginResponderMixin, ViewResponder):
     template_name = 'keg_auth/flash-messages-only.html'
 
     def __init__(self, parent):
-        warnings.warn(OidcAuthenticator.usage_warning, PendingDeprecationWarning, 2)
+        warnings.warn(OidcAuthenticator.usage_warning, DeprecationWarning, 2)
         super().__init__(parent)
 
     def get(self, *args, **kwargs):
@@ -832,7 +1013,7 @@ class OidcLogoutViewResponder(LogoutViewResponder):
     """ OIDC logout requires some extra leg-work, because token gets refreshed server-side"""
 
     def __init__(self, parent):
-        warnings.warn(OidcAuthenticator.usage_warning, PendingDeprecationWarning, 2)
+        warnings.warn(OidcAuthenticator.usage_warning, DeprecationWarning, 2)
         super().__init__(parent)
 
     def get(self):
@@ -881,12 +1062,17 @@ class OidcLogoutViewResponder(LogoutViewResponder):
 
 
 class OidcAuthenticator(LoginAuthenticator):
-    """ Uses OIDC authentication with an oauth provider, validates against keg-auth db"""
+    """ Uses OIDC authentication with an oauth provider, validates against keg-auth db.
+
+    Warning: Deprecated.
+    """
     responder_cls = {
         'login': OidcLoginViewResponder,
         'logout': OidcLogoutViewResponder,
     }
     usage_warning = (
+        'All OIDC objects are now deprecated and will be removed in a future version. Use the '
+        'OAuth authenticator alongside either a password authenticator or a redirect. '
         'flask-oidc formed the core of this authenticator but has not received much maintenance '
         'in the last few years. At this time, the library is depending on deprecated/removed '
         'items in other libraries (such as itsdangerous) and will need to be updated to be '
@@ -894,7 +1080,7 @@ class OidcAuthenticator(LoginAuthenticator):
     )
 
     def __init__(self, app):
-        warnings.warn(self.usage_warning, PendingDeprecationWarning, 2)
+        warnings.warn(self.usage_warning, DeprecationWarning, 2)
 
         from flask_oidc import OpenIDConnect
 
