@@ -55,6 +55,8 @@ class AuthAttemptTests(object):
         if has_attempt_model:
             cls.attempt_ent = flask.current_app.auth_manager.entity_registry.attempt_cls
         cls.client = flask_webtest.TestApp(flask.current_app)
+        cls.group_ent = flask.current_app.auth_manager.entity_registry.group_cls
+        cls.bundle_ent = flask.current_app.auth_manager.entity_registry.bundle_cls
 
     def do_login(self, client, email, password, submit_status=200):
         login_url = flask.url_for(flask.current_app.auth_manager.endpoint('login'))
@@ -82,16 +84,21 @@ class AuthAttemptTests(object):
         False,
         True,
     ])
-    def test_login_attempts_blocked(self, limit, timespan, lockout, create_user):
+    @pytest.mark.parametrize('view_config', [
+        False,
+        True,
+    ])
+    def test_login_attempts_blocked(self, limit, timespan, lockout, create_user, view_config):
         '''
         Test that login attempts get blocked after reaching the failed login attempt
         limit. Login attempts after the lockout period has passed (since the failed attempt
         that caused the lockout) should not be blocked.
         '''
+        config_key = 'LOGIN_' if view_config else ''
         with mock.patch.dict('flask.current_app.config', {
-            'KEGAUTH_LOGIN_ATTEMPT_LIMIT': limit,
-            'KEGAUTH_LOGIN_ATTEMPT_TIMESPAN': timespan,
-            'KEGAUTH_LOGIN_ATTEMPT_LOCKOUT': lockout,
+            f'KEGAUTH_{config_key}ATTEMPT_LIMIT': limit,
+            f'KEGAUTH_{config_key}ATTEMPT_TIMESPAN': timespan,
+            f'KEGAUTH_{config_key}ATTEMPT_LOCKOUT': lockout,
         }):
             # We want to test blocking attempts for existing and non-existing users.
             username = 'foo@bar.com'
@@ -162,13 +169,12 @@ class AuthAttemptTests(object):
 
     @pytest.mark.skipif(not has_attempt_model, reason=has_attempt_skip_reason)
     @mock.patch.dict('flask.current_app.config', {
+        'KEGAUTH_ATTEMPT_LIMIT_ENABLED': False,
         'KEGAUTH_LOGIN_ATTEMPT_LIMIT': 3,
         'KEGAUTH_LOGIN_ATTEMPT_TIMESPAN': 3600,
         'KEGAUTH_LOGIN_ATTEMPT_LOCKOUT': 7200,
     })
-    @mock.patch('flask.current_app.auth_manager.entity_registry._attempt_cls',
-                new_callable=mock.PropertyMock(return_value=None))
-    def test_login_attempts_not_blocked(self, _):
+    def test_login_attempts_not_blocked(self):
         '''
         Test that we do not block any attempts with missing attempt entity.
         '''
@@ -187,6 +193,18 @@ class AuthAttemptTests(object):
         do_test(0, self.login_invalid_flashes)
         do_test(0, self.login_success_flashes, 'pass', 302)
 
+    @mock.patch('flask.current_app.auth_manager.entity_registry._attempt_cls',
+                new_callable=mock.PropertyMock(return_value=None))
+    def test_login_attempts_blocked_but_not_configured(self, _):
+        '''
+        Test that we check for the registered attempt entity when limiting is enabled.
+        '''
+        user = self.user_ent.testing_create(email='foo@bar.com', password='pass')
+        assert self.attempt_ent.query.count() == 0
+
+        with pytest.raises(Exception, match=r'.*attempt entity is not registered.*'):
+            self.do_login(self.client, user.email, 'badpass', 200)
+
     @pytest.mark.skipif(not has_attempt_model, reason=has_attempt_skip_reason)
     @pytest.mark.parametrize('limit, timespan, lockout', [
         (3, 3600, 7200),
@@ -220,6 +238,8 @@ class AuthAttemptTests(object):
             # Create a successful login to reset the attempt counter.
             do_test(login_time, self.login_success_flashes, 'pass', 302)
 
+            self.client.get('/logout')
+
             # We can attempt (limit) more times after a successful login before
             # getting locked out.
             for i in range(0, limit):
@@ -382,13 +402,12 @@ class AuthAttemptTests(object):
 
     @pytest.mark.skipif(not has_attempt_model, reason=has_attempt_skip_reason)
     @mock.patch.dict('flask.current_app.config', {
+        'KEGAUTH_ATTEMPT_LIMIT_ENABLED': False,
         'KEGAUTH_FORGOT_ATTEMPT_LIMIT': 3,
         'KEGAUTH_FORGOT_ATTEMPT_TIMESPAN': 3600,
         'KEGAUTH_FORGOT_ATTEMPT_LOCKOUT': 7200,
     })
-    @mock.patch('flask.current_app.auth_manager.entity_registry._attempt_cls',
-                new_callable=mock.PropertyMock(return_value=None))
-    def test_forgot_attempts_not_blocked(self, _):
+    def test_forgot_attempts_not_blocked(self):
         '''
         Test that we do not block any attempts with missing attempt entity.
         '''
@@ -539,13 +558,12 @@ class AuthAttemptTests(object):
 
     @pytest.mark.skipif(not has_attempt_model, reason=has_attempt_skip_reason)
     @mock.patch.dict('flask.current_app.config', {
+        'KEGAUTH_ATTEMPT_LIMIT_ENABLED': False,
         'KEGAUTH_RESET_ATTEMPT_LIMIT': 2,
         'KEGAUTH_RESET_ATTEMPT_TIMESPAN': 3600,
         'KEGAUTH_RESET_ATTEMPT_LOCKOUT': 7200,
     })
-    @mock.patch('flask.current_app.auth_manager.entity_registry._attempt_cls',
-                new_callable=mock.PropertyMock(return_value=None))
-    def test_reset_pw_attempts_not_blocked(self, _):
+    def test_reset_pw_attempts_not_blocked(self):
         user = self.user_ent.testing_create()
         assert self.attempt_ent.query.count() == 0
 
@@ -1043,6 +1061,126 @@ class AuthTests(AuthAttemptTests):
         # Confirm logout occured
         client.get(self.protected_url, status=302)
 
+    def test_list_user(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get('/users?op(username)=eq&v1(username)=' + user.email)
+        assert resp.pyquery('.grid-header-add-link a').attr('href').startswith('/users/add')
+        assert resp.pyquery('h1').eq(0).text() == 'Users'
+        assert resp.pyquery('.grid-header-add-link').eq(0).text() == 'Create User'
+
+    def test_edit_user(self):
+        user_edit = self.user_ent.testing_create()
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get('/users/{}/edit'.format(user_edit.id))
+        assert resp.pyquery('h1').eq(0).text() == 'Edit User'
+        resp = resp.form.submit()
+
+        assert resp.status_code == 302
+        assert resp.location.endswith('/users')
+        assert resp.flashes == [('success', 'Successfully modified User')]
+
+    def test_delete_user(self):
+        user_delete_id = self.user_ent.testing_create().id
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get('/users/{}/delete'.format(user_delete_id))
+
+        assert resp.status_code == 302
+        assert resp.location.endswith('/users')
+        assert resp.flashes == [('success', 'Successfully removed User')]
+
+        assert not self.user_ent.query.get(user_delete_id)
+
+    def test_list_group(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        group = self.group_ent.testing_create()
+        resp = client.get('/groups?op(name)=eq&v1(name)=' + group.name)
+        assert resp.pyquery('.grid-header-add-link a').attr('href').startswith('/groups/add')
+        assert resp.pyquery('h1').eq(0).text() == 'Groups'
+        assert resp.pyquery('.grid-header-add-link').eq(0).text() == 'Create Group'
+
+    def test_add_group(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get('/groups/add')
+        group_name = randchars()
+        resp.form['name'] = group_name
+        resp = resp.form.submit()
+        assert resp.status_code == 302
+        assert resp.location.endswith('/groups')
+        assert resp.flashes == [('success', 'Successfully created Group')]
+
+        assert self.group_ent.get_by(name=group_name)
+
+    def test_edit_group(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        group = self.group_ent.testing_create()
+        resp = client.get(f'/groups/{group.id}/edit')
+        resp = resp.form.submit()
+        assert resp.status_code == 302
+        assert resp.location.endswith('/groups')
+        assert resp.flashes == [('success', 'Successfully modified Group')]
+
+    def test_delete_group(self):
+        group_delete_id = self.group_ent.testing_create().id
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get(f'/groups/{group_delete_id}/delete')
+
+        assert resp.status_code == 302
+        assert resp.location.endswith('/groups')
+        assert resp.flashes == [('success', 'Successfully removed Group')]
+
+        assert not self.group_ent.query.get(group_delete_id)
+
+    def test_list_bundle(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        bundle = self.bundle_ent.testing_create()
+        resp = client.get('/bundles?op(name)=eq&v1(name)=' + bundle.name)
+        assert resp.pyquery('.grid-header-add-link a').attr('href').startswith('/bundles/add')
+        assert resp.pyquery('h1').eq(0).text() == 'Bundles'
+        assert resp.pyquery('.grid-header-add-link').eq(0).text() == 'Create Bundle'
+
+    def test_add_bundle(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get('/bundles/add')
+        bundle_name = randchars()
+        resp.form['name'] = bundle_name
+        resp = resp.form.submit()
+        assert resp.status_code == 302
+        assert resp.location.endswith('/bundles')
+        assert resp.flashes == [('success', 'Successfully created Bundle')]
+
+        assert self.bundle_ent.get_by(name=bundle_name)
+
+    def test_edit_bundle(self):
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        bundle = self.bundle_ent.testing_create()
+        resp = client.get(f'/bundles/{bundle.id}/edit')
+        resp = resp.form.submit()
+        assert resp.status_code == 302
+        assert resp.location.endswith('/bundles')
+        assert resp.flashes == [('success', 'Successfully modified Bundle')]
+
+    def test_delete_bundle(self):
+        bundle_delete_id = self.bundle_ent.testing_create().id
+        user = self.user_ent.testing_create(permissions=['auth-manage'])
+        client = AuthTestApp(flask.current_app, user=user)
+        resp = client.get(f'/bundles/{bundle_delete_id}/delete')
+
+        assert resp.status_code == 302
+        assert resp.location.endswith('/bundles')
+        assert resp.flashes == [('success', 'Successfully removed Bundle')]
+
+        assert not self.bundle_ent.query.get(bundle_delete_id)
+
 
 @wrapt.decorator
 def user_request(wrapped, instance, args, kwargs):
